# -*- encoding: utf-8 -*-
"""
@File    :   metrics.py
@Time    :   2025/2/25 下午8:47
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""
import os, sys, argparse, numpy as np, itertools
from sklearn.metrics import precision_recall_fscore_support
from logger import write_log as logging


def f_value(y_pred, y_true, ignore=0, vocab=None, name="", is2d=False, verbose=True, very_verbose=False, \
            verbose_return=True):
        """
        Easy method to compute F1 score given results and label lists

        Parameter:
        ----------
        y_pred: list of predictions
        y_true: list of labels
        ignore: ignore some unwanted label, for example, -1
        vocab:  sometimes the labels are digits but meaning some texts,
                vocab replace digits with texts for better presentation
        name:
        is2d:
        verbose: bool, print out a table
        very_verbose: bool, print out a table
        verbose_return: bool, return p,r,f1; otherwise only return f1

        Return:
        -------
        macro and micro f1 or macro and micro p,r,f1
        """
        if is2d:
                y_pred = list(itertools.chain(*y_pred))
                y_true = list(itertools.chain(*y_true))
        labels = [_ for _ in set(y_pred + y_true) if _ != ignore]
        labels.sort()
        ps, rs, fs, _ = precision_recall_fscore_support(y_true, y_pred, labels=labels, average=None)
        micro_p, micro_r, micro_f, _ = precision_recall_fscore_support(y_true, y_pred, labels=labels, average="micro", warn_for=())
        macro_p, macro_r, macro_f, _ = precision_recall_fscore_support(y_true, y_pred, labels=labels, average="macro", warn_for=())
        rows = [["Class", "N", "P", "R", "F", ""]]
        if very_verbose:
                for lab, p, r, f in zip(labels, ps, rs, fs):
                        desc = vocab.get(lab, '') if vocab is not None else ''
                        n = len([_ for _ in y_true if _ == lab])
                        rows.append([lab, n, p, r, f, desc])
        if verbose or very_verbose:
                n = len([_ for _ in y_true if _ != ignore])
                rows.append(["Macro", n, macro_p, macro_r, macro_f, ''])
                rows.append(["Micro", n, micro_p, micro_r, micro_f, ''])
                logging.info(name)
                logging.table(rows)
        if not verbose_return:
                return macro_f, micro_f
        return macro_p, macro_r, macro_f, micro_p, micro_r, micro_f


def dcg_at_k(r, k, method=0):
        """Score is discounted cumulative gain (dcg)
        Relevance is positive real values.  Can use binary
        as the previous methods.
        Example from
        http://www.stanford.edu/class/cs276/handouts/EvaluationNew-handout-6-per.pdf
        >>> r = [3, 2, 3, 0, 0, 1, 2, 2, 3, 0]
        >>> dcg_at_k(r, 1)
        3.0
        >>> dcg_at_k(r, 1, method=1)
        3.0
        >>> dcg_at_k(r, 2)
        5.0
        >>> dcg_at_k(r, 2, method=1)
        4.2618595071429155
        >>> dcg_at_k(r, 10)
        9.6051177391888114
        >>> dcg_at_k(r, 11)
        9.6051177391888114
        Args:
            r: Relevance scores (list or numpy) in rank order
                (first element is the first item)
            k: Number of results to consider
            method: If 0 then weights are [1.0, 1.0, 0.6309, 0.5, 0.4307, ...]
                    If 1 then weights are [1.0, 0.6309, 0.5, 0.4307, ...]
        Returns:
            Discounted cumulative gain
        """
        r = np.asfarray(r)[:k]
        if r.size:
                if method == 0:
                        return r[0] + np.sum(r[1:] / np.log2(np.arange(3, r.size + 2)))  # fix here
                elif method == 1:
                        return np.sum(r / np.log2(np.arange(2, r.size + 2)))
                else:
                        raise ValueError('method must be 0 or 1.')
        return 0.


def ndcg_at_k(r, k, method=0):
        """Score is normalized discounted cumulative gain (ndcg)
        Relevance is positive real values.  Can use binary
        as the previous methods.
        Example from
        http://www.stanford.edu/class/cs276/handouts/EvaluationNew-handout-6-per.pdf
        >>> r = [3, 2, 3, 0, 0, 1, 2, 2, 3, 0]
        >>> ndcg_at_k(r, 1)
        1.0
        >>> r = [2, 1, 2, 0]
        >>> ndcg_at_k(r, 4)
        0.9203032077642922
        >>> ndcg_at_k(r, 4, method=1)
        0.96519546960144276
        >>> ndcg_at_k([0], 1)
        0.0
        >>> ndcg_at_k([1], 2)
        1.0
        Args:
            r: Relevance scores (list or numpy) in rank order
                (first element is the first item)
            k: Number of results to consider
            method: If 0 then weights are [1.0, 1.0, 0.6309, 0.5, 0.4307, ...]
                    If 1 then weights are [1.0, 0.6309, 0.5, 0.4307, ...]
        Returns:
            Normalized discounted cumulative gain
        """
        dcg_max = dcg_at_k(sorted(r, reverse=True), k, method)
        if not dcg_max:
                return 0.
        return dcg_at_k(r, k, method) / dcg_max



# -*- encoding: utf-8 -*-
"""
@File    :   algo.py
@Time    :   2025/2/25 下午8:31
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""
import os, sys, argparse
from tqdm import tqdm
from itertools import groupby, tee
from datetime import timedelta
from collections import Counter
import codecs, json, \
        time, datetime, pickle, \
        re, random, numpy as np, \
        pandas as pd, xml.dom.minidom as xmldom, string
import inspect
import platform
import yaml
import itertools
from pdb import set_trace as stop
from typing import Any, List, Tuple, Union


def dupstr(source, elmt):  # The source may be a list or string.
        elmt_index = []
        s_index = 0
        e_index = len(source)
        while s_index < e_index:
                try:
                        temp = source.index(elmt, s_index, e_index)
                        elmt_index.append(temp)
                        s_index = temp + 1
                except ValueError:
                        break
        return elmt_index


def retgroup(inputlst):
        ''' 返回连续序列中的连续相同项 '''
        groups = [(k, sum(1 for _ in g)) for k, g in groupby(inputlst)]
        cursor = 0
        result = []
        for k, l in groups:
                result.append((k, [cursor, cursor + l - 1]))
                cursor += l
        return result


def find_substr_all(in_string, substr):
        """
        找到不重叠的子串的位置
        :param in_string:
        :param substr:
        :return:
        """
        out_pos = []
        if not in_string or not substr:
                return []
        start = 0
        is_found = in_string[start:].find(substr)
        while is_found != -1:
                assert in_string[start + is_found:start + is_found + len(substr)] == substr
                out_pos.append((start + is_found, start + is_found + len(substr)))
                start = start + is_found + len(substr)
                is_found = in_string[start:].find(substr)
        return out_pos


def argmin(lst):
        return min(range(len(lst)), key=lst.__getitem__)


def argmax(lst):
        return max(range(len(lst)), key=lst.__getitem__)


def groupby_continuous_index(index_list):
        from itertools import groupby
        func = lambda x: x[1] - x[0]
        output = []
        for k, g in groupby(enumerate(index_list), func):
                l1 = [j for i, j in g]  # 连续数字的列表
                if len(l1) > 1:
                        scop = list(range(min(l1), max(l1) + 1))  # 将连续数字范围用"-"连接
                else:
                        scop = l1[0]
                output.append(scop)
        return output


# NLP

def punctuation_split(txt):
        pattern = r',|\.|/|;|\'|`|\[|\]|<|>|\?|:|"|\{|\}|\~|!|@|#|\$|%|\^|&|\(|\)|-|=|\_|\+|，|。|、|；|‘|’|【|】|·|！| |…|（|）'
        return re.split(pattern, txt)


def strip_margin(text):
        return re.sub('\n[ \t]*\|', '\n', text)


def strip_heredoc(text):
        indent = len(min(re.findall('\n[ \t]*(?=\S)', text) or ['']))
        pattern = r'\n[ \t]{%d}' % (indent - 1)
        return re.sub(pattern, '\n', text)


def getRidOfPunc(inputs):
        """
        [usage]: 去除文本中的标点符号，只提取中文英文数字
        """
        cn_text = re.compile(r'[\u4e00-\u9fa5_a-zA-Z0-9]{2,50}', re.IGNORECASE)
        tokens = cn_text.findall(inputs)
        return ''.join(tokens)


def getZhChar(inputs):
        """
        [usage]: 只提取中文
        """
        cn_text = re.compile(r'[\u4e00-\u9fa5]{1,50}', re.IGNORECASE)
        tokens = cn_text.findall(inputs)
        return ''.join(tokens)


def getEnChar(inputs):
        """
        [usage]: 只提取英文
        """
        cn_text = re.compile(r'[a-zA-Z]{1,50}', re.IGNORECASE)
        tokens = cn_text.findall(inputs)
        return ''.join(tokens)


def getNumChar(inputs):
        """
        [usage]: 只提取数字
        """
        cn_text = re.compile(r'[0-9]{1,50}', re.IGNORECASE)
        tokens = cn_text.findall(inputs)
        return ''.join(tokens)


def extract_room_expr(inputs):
    """
    [usage]: 提取XXX室（门牌号表达）
    """
    room_rg = re.compile("[a-zA-Z0-9\-甲乙丙丁戊己庚戌东南西北一二三四五六七八九]+室", re.IGNORECASE)
    expr_list = room_rg.findall(inputs)
    return expr_list







# -*- encoding: utf-8 -*-
"""
@File    :   logger.py
@Time    :   2025/2/25 下午8:46
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""

# here put the import lib
import os,sys,argparse,logging
from baseio import jdir
def custom_create(path, name):
    file_name = f"log.{name}.txt"
    jdir(path)
    log_file = os.path.join(path, file_name)
    formatter = logging.Formatter(fmt='%(asctime)s %(message)s', datefmt='%m/%d %H:%M:%S')
    logger = logging.getLogger(__name__)
    file_handler = logging.FileHandler(log_file)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    screen_handler = logging.StreamHandler()
    screen_handler.setFormatter(formatter)
    logger.addHandler(screen_handler)
    logger.setLevel(logging.INFO)
    info(f"LOGGING >>> {log_file}")

    # info("CUDA_VISIBLE_DEVICES:", os.environ["CUDA_VISIBLE_DEVICES"])
    # command("nvidia-smi")
def create(path, train):
    file_name = 'log.train.txt' if train else 'log.test.txt'
    if not os.path.exists(path):
        if train:
            os.makedirs(path)
        else:
            raise FileNotFoundError(path)
    log_file = os.path.join(path, file_name)
    formatter = logging.Formatter(fmt='%(asctime)s %(message)s', datefmt='%m/%d %H:%M:%S')
    logger = logging.getLogger(__name__)
    file_handler = logging.FileHandler(log_file)
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    screen_handler = logging.StreamHandler()
    screen_handler.setFormatter(formatter)
    logger.addHandler(screen_handler)
    logger.setLevel(logging.INFO)
    info(f"LOGGING >>> {log_file}")

    # info("CUDA_VISIBLE_DEVICES:", os.environ["CUDA_VISIBLE_DEVICES"])
    # command("nvidia-smi")

def command(cmd):
    lines = os.popen(cmd).readlines()
    lines = ''.join(lines)
    info(cmd + '\n' + lines)

def info(*msg):
    msg = ' '.join([str(_) for _ in msg])
    logging.getLogger(__name__).log(logging.INFO, msg)

def warn(*msg):
    msg = ' '.join([str(_) for _ in msg])
    logging.getLogger(__name__).log(logging.WARN, msg)

def table(rows):
    rows = [list(row) for row in rows]
    num_col = len(rows[0])
    col_lengths = [0 for _ in range(num_col)]
    for row in rows:
        for j, item in enumerate(row):
            item = to_string(item)
            row[j] = item
            if len(item) > col_lengths[j]:
                col_lengths[j] = len(item)
    outputs = []
    for i, row in enumerate(rows):
        output = ["|"]
        for j, item in enumerate(row):
            item = to_string(item)
            padding = ' ' * (col_lengths[j] - len(item))
            output.append(item + padding)
        info('\t'.join(output))
        outputs.append(output)
    return outputs

def to_string(x):
    import numpy as np
    if type(x) in [float, np.float32, np.float64]:
        return f"{x:.3f}"
    else:
        return str(x)



# -*- encoding: utf-8 -*-
"""
@File    :   image.py
@Time    :   2025/2/25 下午8:35
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""
import os, sys, argparse
from tqdm import tqdm
from itertools import groupby, tee
from datetime import timedelta
from collections import Counter
import codecs, json, \
        time, datetime, pickle, \
        re, random, numpy as np, \
        pandas as pd, xml.dom.minidom as xmldom, string
import inspect
import platform
import yaml
import itertools
from pdb import set_trace as stop
from typing import Any, List, Tuple, Union
from PIL import Image
import numpy as np
import matplotlib.pyplot as plt
import io
import cv2
import base64



def png2base64(png_path):
    import base64
    with open(png_path,"rb") as f:
        # b64encode是编码，b64decode是解码
        base64_data = base64.b64encode(f.read())
        # base64.b64decode(base64data)
    return base64_data


def readImgPIL(path, cvt=False, mode="RGB"):
        if cvt:
                assert mode in ["RGB", "RGBA", "L"], f"convert mode param error, input is : {mode}"
                return Image.open(path).convert(mode)
        else:
                return Image.open(path)


def saveImg(img, path, cvt=False, mode="RGB"):
        if cvt:
                assert mode in ["RGB", "RGBA", "L"], f"convert mode param error, input is : {mode}"
                im_save = Image.fromarray(img).convert(mode)
        else:
                im_save = Image.fromarray(img)
        im_save.save(path)


def matrix2image(mat, mode="RGB"):
        """ if inputs is list of numpy matrices, use this to generate pil image for model inputs"""
        return Image.fromarray(mat).convert(mode)


def readImg(path, cvt=False, mode="RGB"):
        if cvt:
                assert mode in ["RGB", "RGBA", "L"], f"convert mode param error, input is : {mode}"
                return np.array(Image.open(path).convert(mode))
        else:
                return np.array(Image.open(path))


def RGB2L(img):
        return np.array(Image.fromarray(img).convert("L"))


def L2RGB(img):
        return np.array(Image.fromarray(img).convert("RGB"))


def showImgs(imglist, shape):
        plt.figure(figsize=(15, 15))
        for i in range(len(imglist)):
                plt.subplot(shape[0], shape[1], i + 1)
                plt.imshow(imglist[i])
        plt.show()


def check_pixel_set(img):
        print(set(img.reshape(-1).tolist()))


'''
###############################################
'''

'''
########  Image Color  ######
'''


def change_color(img, old_color, new_color):
        """
        RGB:
          ---------------------------------------------------------
    　    |Pink	粉红	#FFC0CB	255,192,203
          |LightPink 浅粉红 #FFB6C1 255,182,193
    　    |Crimson	猩红	#DC143C	220,20,60
    　    |LavenderBlush	脸红的淡紫色	#FFF0F5	255,240,245
    　    |PaleVioletRed	苍白的紫罗兰红色	#DB7093	219,112,147
    　    |HotPink	热情的粉红	#FF69B4	255,105,180
    　    |DeepPink	深粉色	#FF1493	255,20,147
    　    |MediumVioletRed	适中的紫罗兰红色	#C71585	199,21,133
    　    |Orchid	兰花的紫色	#DA70D6	218,112,214
    　    |Thistle	蓟	#D8BFD8	216,191,216
    　    |plum	李子	#DDA0DD	221,160,221
    　    |Violet	紫罗兰	#EE82EE	238,130,238
    　    |Magenta	洋红	#FF00FF	255,0,255
    　    |Fuchsia	灯笼海棠(紫红色)	#FF00FF	255,0,255
    　    |DarkMagenta	深洋红色	#8B008B	139,0,139
    　    |Purple	紫色	#800080	128,0,128
    　    |MediumOrchid	适中的兰花紫	#BA55D3	186,85,211
    　    |DarkVoilet	深紫罗兰色	#9400D3	148,0,211
    　    |DarkOrchid	深兰花紫	#9932CC	153,50,204
    　    |Indigo	靛青	#4B0082	75,0,130
    　    |BlueViolet	深紫罗兰的蓝色	#8A2BE2	138,43,226
    　    |MediumPurple	适中的紫色	#9370DB	147,112,219
    　    |MediumSlateBlue	适中的板岩暗蓝灰色	#7B68EE	123,104,238
    　    |SlateBlue	板岩暗蓝灰色	#6A5ACD	106,90,205
    　    |DarkSlateBlue	深岩暗蓝灰色	#483D8B	72,61,139
    　    |Lavender	熏衣草花的淡紫色	#E6E6FA	230,230,250
    　    |GhostWhite	幽灵的白色	#F8F8FF	248,248,255
    　    |Blue	纯蓝	#0000FF	0,0,255
    　    |MediumBlue	适中的蓝色	#0000CD	0,0,205
    　    |MidnightBlue	午夜的蓝色	#191970	25,25,112
    　    |DarkBlue	深蓝色	#00008B	0,0,139
    　    |Navy	海军蓝	#000080	0,0,128
    　    |RoyalBlue	皇家蓝	#4169E1	65,105,225
    　    |CornflowerBlue	矢车菊的蓝色	#6495ED	100,149,237
    　    |LightSteelBlue	淡钢蓝	#B0C4DE	176,196,222
    　    |LightSlateGray	浅石板灰	#778899	119,136,153
    　    |SlateGray	石板灰	#708090	112,128,144
    　    |DoderBlue	道奇蓝	#1E90FF	30,144,255
    　    |AliceBlue	爱丽丝蓝	#F0F8FF	240,248,255
    　    |SteelBlue	钢蓝	#4682B4	70,130,180
    　    |LightSkyBlue	淡蓝色	#87CEFA	135,206,250
    　    |SkyBlue	天蓝色	#87CEEB	135,206,235
    　    |DeepSkyBlue	深天蓝	#00BFFF	0,191,255
    　    |LightBLue	淡蓝	#ADD8E6	173,216,230
    　    |PowDerBlue	火药蓝	#B0E0E6	176,224,230
    　    |CadetBlue	军校蓝	#5F9EA0	95,158,160
    　    |Azure	蔚蓝色	#F0FFFF	240,255,255
    　    |LightCyan	淡青色	#E1FFFF	225,255,255
    　    |PaleTurquoise	苍白的绿宝石	#AFEEEE	175,238,238
    　    |Cyan	青色	#00FFFF	0,255,255
    　    |Aqua	水绿色	#00FFFF	0,255,255
    　    |DarkTurquoise	深绿宝石	#00CED1	0,206,209
    　    |DarkSlateGray	深石板灰	#2F4F4F	47,79,79
    　    |DarkCyan	深青色	#008B8B	0,139,139
    　    |Teal	水鸭色	#008080	0,128,128
    　    |MediumTurquoise	适中的绿宝石	#48D1CC	72,209,204
    　    |LightSeaGreen	浅海洋绿	#20B2AA	32,178,170
    　    |Turquoise	绿宝石	#40E0D0	64,224,208
    　    |Auqamarin	绿玉/碧绿色	#7FFFAA	127,255,170
    　    |MediumAquamarine	适中的碧绿色	#00FA9A	0,250,154
    　    |MediumSpringGreen	适中的春天的绿色	#F5FFFA	245,255,250
    　    |MintCream	薄荷奶油	#00FF7F	0,255,127
    　    |SpringGreen	春天的绿色	#3CB371	60,179,113
    　    |SeaGreen	海洋绿	#2E8B57	46,139,87
    　    |Honeydew	蜂蜜	#F0FFF0	240,255,240
    　    |LightGreen	淡绿色	#90EE90	144,238,144
    　    |PaleGreen	苍白的绿色	#98FB98	152,251,152
    　    |DarkSeaGreen	深海洋绿	#8FBC8F	143,188,143
    　    |LimeGreen	酸橙绿	#32CD32	50,205,50
    　    |Lime	酸橙色	#00FF00	0,255,0
    　    |ForestGreen	森林绿	#228B22	34,139,34
    　    |Green	纯绿	#008000	0,128,0
    　    |DarkGreen	深绿色	#006400	0,100,0
    　    |Chartreuse	查特酒绿	#7FFF00	127,255,0
    　    |LawnGreen	草坪绿	#7CFC00	124,252,0
    　    |GreenYellow	绿黄色	#ADFF2F	173,255,47
    　    |OliveDrab	橄榄土褐色	#556B2F	85,107,47
    　    |Beige	米色(浅褐色)	#6B8E23	107,142,35
    　    |LightGoldenrodYellow	浅秋麒麟黄	#FAFAD2	250,250,210
    　    |Ivory	象牙	#FFFFF0	255,255,240
    　    |LightYellow	浅黄色	#FFFFE0	255,255,224
    　    |Yellow	纯黄	#FFFF00	255,255,0
    　    |Olive	橄榄	#808000	128,128,0
    　    |DarkKhaki	深卡其布	#BDB76B	189,183,107
    　    |LemonChiffon	柠檬薄纱	#FFFACD	255,250,205
    　    |PaleGodenrod	灰秋麒麟	#EEE8AA	238,232,170
    　    |Khaki	卡其布	#F0E68C	240,230,140
    　    |Gold	金	#FFD700	255,215,0
    　    |Cornislk	玉米色	#FFF8DC	255,248,220
    　    |GoldEnrod	秋麒麟	#DAA520	218,165,32
    　    |FloralWhite	花的白色	#FFFAF0	255,250,240
    　    |OldLace	老饰带	#FDF5E6	253,245,230
    　    |Wheat	小麦色	#F5DEB3	245,222,179
    　    |Moccasin	鹿皮鞋	#FFE4B5	255,228,181
    　    |Orange	橙色	#FFA500	255,165,0
    　    |PapayaWhip	番木瓜	#FFEFD5	255,239,213
    　    |BlanchedAlmond	漂白的杏仁	#FFEBCD	255,235,205
    　    |NavajoWhite	Navajo白	#FFDEAD	255,222,173
    　    |AntiqueWhite	古代的白色	#FAEBD7	250,235,215
    　    |Tan	晒黑	#D2B48C	210,180,140
    　    |BrulyWood	结实的树	#DEB887	222,184,135
    　    |Bisque	(浓汤)乳脂,番茄等	#FFE4C4	255,228,196
    　    |DarkOrange	深橙色	#FF8C00	255,140,0
    　    |Linen	亚麻布	#FAF0E6	250,240,230
    　    |Peru	秘鲁	#CD853F	205,133,63
    　    |PeachPuff	桃色	#FFDAB9	255,218,185
    　    |SandyBrown	沙棕色	#F4A460	244,164,96
    　    |Chocolate	巧克力	#D2691E	210,105,30
    　    |SaddleBrown	马鞍棕色	#8B4513	139,69,19
    　    |SeaShell	海贝壳	#FFF5EE	255,245,238
    　    |Sienna	黄土赭色	#A0522D	160,82,45
    　    |LightSalmon	浅鲜肉(鲑鱼)色	#FFA07A	255,160,122
    　    |Coral	珊瑚	#FF7F50	255,127,80
    　    |OrangeRed	橙红色	#FF4500	255,69,0
    　    |DarkSalmon	深鲜肉(鲑鱼)色	#E9967A	233,150,122
    　    |Tomato	番茄	#FF6347	255,99,71
    　    |MistyRose	薄雾玫瑰	#FFE4E1	255,228,225
    　    |Salmon	鲜肉(鲑鱼)色	#FA8072	250,128,114
    　    |Snow	雪	#FFFAFA	255,250,250
    　    |LightCoral	淡珊瑚色	#F08080	240,128,128
    　    |RosyBrown	玫瑰棕色	#BC8F8F	188,143,143
    　    |IndianRed	印度红	#CD5C5C	205,92,92
    　    |Red	纯红	#FF0000	255,0,0
    　    |Brown	棕色	#A52A2A	165,42,42
    　    |FireBrick	耐火砖	#B22222	178,34,34
    　    |DarkRed	深红色	#8B0000	139,0,0
    　    |Maroon	栗色	#800000	128,0,0
    　    |White	纯白	#FFFFFF	255,255,255
    　    |WhiteSmoke	白烟	#F5F5F5	245,245,245
    　    |Gainsboro	Gainsboro	#DCDCDC	220,220,220
    　    |LightGrey	浅灰色	#D3D3D3	211,211,211
    　    |Silver	银白色	#C0C0C0	192,192,192
    　    |DarkGray	深灰色	#A9A9A9	169,169,169
    　    |Gray	灰色	#808080	128,128,128
    　    |DimGray	暗淡的灰色	#696969	105,105,105
    　    |Black	纯黑	#000000	0,0,0
          ---------------------------------------------------------
        img: 二值化后的图像
        old_color: 旧颜色
        new_color: 新颜色
        """
        new_img = []
        for row in img:
                new_row = []
                for col in row:
                        if '_'.join([str(x) for x in col]) == '_'.join([str(x) for x in old_color]):
                                new_row.append(new_color)
                        else:
                                new_row.append(col)
                new_img.append(new_row)
        new_img = np.array(new_img, dtype=np.uint8)
        return new_img


'''
###############################################
'''

'''
########  Image to Bytes  ######
'''


def numpy2bytes(im, format, image_depth):
        img = Image.fromarray(im).convert(image_depth)
        with io.BytesIO() as image_bytes:
                img.save(image_bytes, format=format)
                image_bytes = image_bytes.getvalue()
        return image_bytes


def bytes2base64(image_bytes):
        image_base64 = base64.b64encode(image_bytes)
        return image_base64


def numpy2base64(im, format='png', image_depth='L', toutf8=True):
        image_bytes = numpy2bytes(im, format, image_depth)
        image_base64 = bytes2base64(image_bytes)
        if toutf8:
                image_base64 = image_base64.decode(encoding='utf-8')
        return image_base64


def base64Tobytes(image_base64):
        image_bytes = base64.b64decode(image_base64)
        return image_bytes


def bytes2numpy(image_bytes):
        img = np.array(Image.open(io.BytesIO(image_bytes)))
        return img


def base64ToNumpy(image_base64):
        image_bytes = base64Tobytes(image_base64)
        img = bytes2numpy(image_bytes)
        return img


def test_image_to_bytes():
        im = np.random.randint(0, 256, (12, 12,)).astype(np.uint8)
        cv2.imwrite('test_random.png', im)
        image_base64 = numpy2base64(im)
        print('base64:\n', image_base64)
        im2 = base64ToNumpy(image_base64)
        cv2.imwrite('test_from_base64.png', im2)


# -*- encoding: utf-8 -*-
"""
@File    :   sys.py
@Time    :   2025/2/25 下午8:22
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""
import os, sys, argparse
from tqdm import tqdm
from itertools import groupby, tee
from datetime import timedelta
from collections import Counter
import codecs, json, \
        time, datetime, pickle, \
        re, random, numpy as np, \
        pandas as pd, xml.dom.minidom as xmldom, string
import inspect
import platform
import yaml
import itertools
from pdb import set_trace as stop
from typing import Any, List, Tuple, Union


def isMacOS():
        return 'Darwin' in platform.platform()


def shell_run(*args):
        for command in [*args]:
                os.system(f"{command}")


def end():
        exit(0)
        clearDir()


def use_gpu(gpu_id):
        os.environ["CUDA_VISIBLE_DEVICES"] = f"{gpu_id}"


def get_FileSize(filePath):
    '''获取文件的大小,结果保留两位小数，单位为MB'''
    filePath = unicode(filePath,'utf8')
    fsize = os.path.getsize(filePath)
    fsize = fsize/float(1024*1024)
    return round(fsize,2)



'''获取文件的访问时间'''
def get_FileAccessTime(filePath):
    filePath = unicode(filePath,'utf8')
    t = os.path.getatime(filePath)
    return TimeStampToTime(t)


'''获取文件的创建时间'''
def get_FileCreateTime(filePath):
    filePath = unicode(filePath,'utf8')
    t = os.path.getctime(filePath)
    return TimeStampToTime(t)


'''获取文件的修改时间'''
def get_FileModifyTime(filePath):
    filePath = unicode(filePath,'utf8')
    t = os.path.getmtime(filePath)
    return TimeStampToTime(t)

def merge_txt(data_dir, out_dir, out_name):
    print(f"[run] cat {data_dir}/*.txt > {out_dir}/{out_name}.txt")
    os.system(f"cat {data_dir}/*.txt > {out_dir}/{out_name}.txt")


def merge_file(data_dir, out_dir, out_name, appendix='txt'):
    print(f"[run] cat {data_dir}/* > {out_dir}/{out_name}.{appendix}")
    os.system(f"cat {data_dir}/* > {out_dir}/{out_name}.{appendix}")


def split_file(file_path, split_mode='l', subsize=500, idx_size=4, suffix='txt',output_dir=None):
    split_cmd = "gsplit" if isMacOS() else "split"
    if output_dir is None:
        target_file_dir = os.path.dirname(file_path)
        target_file_name = file_path.split('/')[-1].split('.')[0]
        output_dir = jdir(jpath(target_file_dir, f"{target_file_name}_splits"))
    print(f"[run] {split_cmd} -{split_mode} {subsize} -da {idx_size} {file_path} {output_dir}/data_ --additional-suffix=.{suffix}")
    os.system(f"{split_cmd} -{split_mode} {subsize} -da {idx_size} {file_path} {output_dir}/data_ --additional-suffix=.{suffix}")







# -*- encoding: utf-8 -*-
"""
@File    :   baseio.py
@Time    :   2025/2/25 下午8:21
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""

import os, sys, argparse
from tqdm import tqdm
from itertools import groupby, tee
from datetime import timedelta
from collections import Counter
import codecs, json, \
        time, datetime, pickle, \
        re, random, numpy as np, \
        pandas as pd, xml.dom.minidom as xmldom, string
import inspect
import platform
import yaml
import itertools
from pdb import set_trace as stop
from typing import Any, List, Tuple, Union


class wixio(object):
        def __init__(self, data=None, len=None, doc=None, info=None, details=None):
                """
                Data structure for general data loaded from various files.

                Parameter:
                ----------
                r: Axis-angle rotation tensor of shape [batch_size, 1, 3].

                Return:
                -------
                Rotation matrix of shape [batch_size, 3, 3].
                """
                self.data = data
                self.len = len
                self.doc = doc
                self.info = info
                self.details = details

        @classmethod
        def tojson(cls, data, path, encoding='utf-8'):
                with codecs.open(path, 'w', encoding=encoding) as f:
                        json.dump(data, f, ensure_ascii=False)
                        print(f'Finished dumping json file at {path}!')

        @classmethod
        def totxt(cls, data: list, path, encoding='utf-8'):
                assert isinstance(data, list)
                with codecs.open(path, 'w', encoding=encoding) as f:
                        for item in data:
                                f.write(item + '\n')
                print(f'Finished dumping txt file at {path}!')

        @classmethod
        def tocsv(cls, data: dict, path, encoding='utf-8'):
                assert isinstance(data, dict)
                df = pd.DataFrame(data)
                df.to_csv(path, sep=',', header=True, index=False)
                print(f"Finished dumping csv file at {path}!")

        @classmethod
        def toxlsx(cls, data: dict, path, encoding='utf-8'):
                assert isinstance(data, dict)
                df = pd.DataFrame(data)
                df.to_excel(path, engine='xlsxwriter', index=False, header=True)
                print(f"Finished dumping xlsx file at {path}!")

        @classmethod
        def savepkl(cls, data, path):
                with open(path, 'wb') as pklfile:
                        pickle.dump(data, pklfile)
                print(f'Finished dumping pkl file at {path}!')

        @classmethod
        def read(cls, path, split_mark='\t', encoding='utf-8', sheet_id=0, dform=None, **kwargs):
                if path.endswith('.txt'):
                        new_lines = []
                        size = 0
                        with open(path, 'r', encoding=encoding, errors='ignore') as file:
                                lines = file.readlines()
                                size = len(lines)
                                for i in range(size):
                                        line = lines[i].strip().split(split_mark)
                                        if len(line) == 1: line = line[0]
                                        new_lines.append(line)
                        return cls(data=new_lines, len=size, doc=path, info='txt')
                elif path.endswith('.json'):
                        with open(path, 'r', encoding=encoding) as f:
                                data = json.loads(f.read())
                                size = len(data)
                        return cls(data=data, len=size, doc=path, info='json')
                elif path.endswith('.xml'):
                        domtree = xmldom.parse(path)
                        rootnode = domtree.documentElement
                        instruction = ("XML文件读取后,采用以下函数读取标签:\n"
                                       "对于一个为'paragraph'的标签: nodes_paragraph = rootnode.getElementsByTagName('paragraph')\n"
                                       "之后通过enumerate遍历里面的内容:\n"
                                       "for idx, node in enumerate(nodes_paragraph): ......\n")
                        return cls(data=rootnode, len=0, doc=path, info=instruction)
                elif path.endswith('.csv'):
                        f = open(path, 'r', encoding=encoding, errors='ignore')
                        main_content = pd.DataFrame(pd.read_csv(f, header=0, low_memory=False))
                        main_content = main_content.where(main_content.notnull(), 'NAN')
                        info = ("注意:\n"
                                "读取的DataFrame经过预处理,将所有空白项填充了字符串 'NAN' \n"
                                "需要调用哪列,就 data.lines[列名]\n")
                        return cls(data=main_content, len=len(main_content), doc=path, info=info)
                elif path.endswith('.xlsx'):
                        import openpyxl
                        # 读取xlsx文件
                        wb = openpyxl.load_workbook(path)
                        ws = wb.worksheets[sheet_id]
                        # sheet页属性：表名、最大行数、最大列数
                        info = 'titles: {}, max_row: {}, max_column: {}'.format(ws.title, ws.max_row, ws.max_column)
                        columns = ws.columns
                        all_data = []
                        for col in columns:
                                a_col = []
                                for item in col: a_col.append(item.value)
                                all_data.append(a_col)
                        headers = [col[0] for col in all_data]
                        return cls(data=all_data, len=len(all_data[0]), doc=path, info=info, details={'header': headers})
                elif path.endswith('.pkl'):
                        f = open(path, 'rb')
                        content = pickle.load(f)
                        return cls(data=content, len=len(content), doc=path, info='pkl')
                elif path.endswith(".yaml"):
                        f = open(path, 'r', encoding='utf-8')
                        cont = f.read()
                        fname = path.split('/')[-1].split('.')[0]
                        res = yaml.load(cont, yaml.FullLoader)
                        attri_len = len(res.keys())
                        all_attributes = '\t'.join([f"({idx}) {x}" for idx, x in enumerate(list(res.keys()))])
                        res = type(f"{fname}", (), res)  # meta class
                        if dform is None or dform == 'cls':
                                return cls(data=res, len=attri_len, doc=path, info=f'yaml\n[Attributes]: {all_attributes}')
                        if dform == 'dict':
                                res_dict = {}
                                for k, v in res.__dict__.items():
                                        if not k.startswith("__"):
                                                res_dict[k] = res.__dict__.get(k, v)
                                return cls(data=res_dict, len=attri_len, doc=path, info=f'yaml\n[Attributes]: {all_attributes}')

        @classmethod
        def rm_nan(cls, df):
                assert isinstance(df, pd.DataFrame), f"input is not pd.DataFrame"
                return df.where(df.notnull(), 'NAN')





def frame_save(data:list, title:str, info:str, datasize=None):
    generate_date = datetime.date.today()
    generate_date.isoformat()
    generate_date = str(generate_date)
    if datasize is None:
        return {'title':title, 'date':generate_date, 'info':info, 'size':f"{len(data)}", 'content':data}
    else:
        return {'title':title, 'date':generate_date, 'info':info, 'size':f"{datasize}", 'content':data}





# -*- encoding: utf-8 -*-
"""
@File    :   time.py
@Time    :   2025/2/25 下午8:36
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""
import os, sys, argparse
from tqdm import tqdm
from itertools import groupby, tee
from datetime import timedelta
from collections import Counter
import codecs, json, \
        time, datetime, pickle, \
        re, random, numpy as np, \
        pandas as pd, xml.dom.minidom as xmldom, string
import inspect
import platform
import yaml
import itertools
from pdb import set_trace as stop
from typing import Any, List, Tuple, Union


def get_time_dif(start_time):
        """获取已使用时间"""
        end_time = time.time()
        time_dif = end_time - start_time
        return timedelta(seconds=int(round(time_dif)))


class msTimer(object):
        @staticmethod
        def set():
                t1 = datetime.datetime.now().microsecond
                t3 = time.mktime(datetime.datetime.now().timetuple())
                return t1, t3

        @staticmethod
        def get(t1, t3, out=False):
                t2 = datetime.datetime.now().microsecond
                t4 = time.mktime(datetime.datetime.now().timetuple())
                strTime = ((t4 - t3) * 1000 + (t2 - t1) / 1000)
                print(f"\ntime passed:{strTime:.3f}ms\n")
                if out:
                        return strTime


def TimeStampToTime(timestamp):
        timeStruct = time.localtime(timestamp)
        return time.strftime('%Y-%m-%d %H:%M:%S', timeStruct)


class GetDateStr:
        @staticmethod
        def time_now():
                return datetime.datetime.now()

        @staticmethod
        def get_x_days_ago(x):
                """
                Get the string of date of x days ago, forms like '20220101'

                Parameter:
                ----------
                x: int, determine a days number

                Return:
                -------
                string of the date of x days ago
                """
                now = datetime.datetime.now()
                return (now - datetime.timedelta(days=x)).strftime("%Y%m%d")

        @staticmethod
        def get_today():
                """
                [usage]:
                    [input]:
                        * null
                    [output]:
                        * yesterday: String, 20200801
                """
                now = datetime.datetime.now()
                return now.strftime("%Y%m%d")

        @staticmethod
        def compute_day_differs(target_date):  # target_date = "20210101"
                """
                Rodrigues' rotation formula that turns axis-angle tensor into rotation
                matrix in a batch-ed manner.

                Parameter:
                ----------
                r: Axis-angle rotation tensor of shape [batch_size, 1, 3].

                Return:
                -------
                Rotation matrix of shape [batch_size, 3, 3].
                """
                examine_days_ago = datetime.date.today().__sub__(datetime.date(int(target_date[:4]), int(target_date[4:6]), int(target_date[6:]))).days
                return examine_days_ago

        @staticmethod
        def compute_date_differs(date1, date2):  # target_date = "20210101"
                """
                Rodrigues' rotation formula that turns axis-angle tensor into rotation
                matrix in a batch-ed manner.

                Parameter:
                ----------
                r: Axis-angle rotation tensor of shape [batch_size, 1, 3].

                Return:
                -------
                Rotation matrix of shape [batch_size, 3, 3].
                """
                differ_days = datetime.date(int(date2[:4]), int(date2[4:6]), int(date2[6:])).__sub__(datetime.date(int(date1[:4]), int(date1[4:6]), int(date1[6:]))).days
                return differ_days

        @staticmethod
        def get_x_days_ago_from_target(target_dt, days):
                """
                Rodrigues' rotation formula that turns axis-angle tensor into rotation
                matrix in a batch-ed manner.

                Parameter:
                ----------
                target_dt: a target date, form like '20220103'
                days, an int, represent N days ago from target_dt, if set to 2, then the result is '20220101'

                Return:
                -------
                dt string
                """
                assert isinstance(target_dt, str), f"target_dt should be string, instead got {target_dt}"
                days = int(days) if not isinstance(days, int) else days
                return GetDateStr.get_x_days_ago(days + GetDateStr.compute_day_differs(target_dt))

        @staticmethod
        def ts_to_hours(ts):
                """
                Rodrigues' rotation formula that turns axis-angle tensor into rotation
                matrix in a batch-ed manner.

                Parameter:
                ----------
                r: Axis-angle rotation tensor of shape [batch_size, 1, 3].

                Return:
                -------
                Rotation matrix of shape [batch_size, 3, 3].
                """
                time_struct = time.localtime(ts)
                time_result = f"{time_struct.tm_year}.{time_struct.tm_mon}.{time_struct.tm_mday}-{time_struct.tm_hour}:{time_struct.tm_min}:{time_struct.tm_sec}-weekday={time_struct.tm_wday}.yearday={time_struct.tm_yday}"
                time_flag = "unknown"

                if 6 <= time_struct.tm_hour < 12:
                        time_flag = "morning"
                if 12 <= time_struct.tm_hour < 15:
                        time_flag = "noon"
                if 15 <= time_struct.tm_hour < 18:
                        time_flag = "afternoon"
                if 18 <= time_struct.tm_hour < 21:
                        time_flag = "evening"
                if 21 <= time_struct.tm_hour < 24 or 0 <= time_struct.tm_hour < 6:
                        time_flag = "night"

                return time_flag, time_result


def TimeStampToTime(timestamp):
        timeStruct = time.localtime(timestamp)
        return time.strftime('%Y-%m-%d %H:%M:%S', timeStruct)


def date_range(start_date, end_date):
        '''
        dates = date_range('20210701','20210705')
        for date in dates:
            print(date)
        '''
        ds = datetime.datetime.strptime(start_date, '%Y%m%d')
        de = datetime.datetime.strptime(end_date, '%Y%m%d')
        delta = (de - ds).days
        dates = [ds + datetime.timedelta(days=i) for i in range(delta)]
        dates = [date.strftime('%Y%m%d') for date in dates]
        return dates



# -*- encoding: utf-8 -*-
"""
@File    :   format.py
@Time    :   2025/2/25 下午8:29
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""


class EasyDict(dict):
        """Convenience class that behaves like a dict but allows access with the attribute syntax."""

        def __getattr__(self, name: str) -> Any:
                try:
                        return self[name]
                except KeyError:
                        raise AttributeError(name)

        def __setattr__(self, name: str, value: Any) -> None:
                self[name] = value

        def __delattr__(self, name: str) -> None:
                del self[name]


def jsondump(x):
        """
        adjusted json.dumps according to
        https://blog.csdn.net/weixin_39561473/article/details/123227500
        param
        ------
        x: dict, input

        return
        ------
        res: string, json.dumps dict string
        """

        def default_dump(obj):
                """Convert numpy classes to JSON serializable objects."""
                if isinstance(obj, (np.integer, np.floating, np.bool_)):
                        return obj.item()
                elif isinstance(obj, np.ndarray):
                        return obj.tolist()
                else:
                        return obj

        res = json.dumps(x, ensure_ascii=False, default=default_dump)
        return res


def dict2class(dct, clsname='default_cls'):
        """
        Fasr converting a input dict to be a python Class

        Parameter:
        ----------
        dct: dict, any dict
        clsname: string, determine the class name for the new class

        Return:
        -------
        res: class, a new class having attributes from dict keys
        attrs: list, a list containing all attribute names
        """
        res = type(f"{clsname}", (), dct)
        attrs = list(res.__dict__.keys())
        return res, attrs


def topct(x, total):
        return round(x / total, ndigits=4) * 100


def limnum(x, out_float=1):
        """
        limit number
        """
        if out_float:
                return float(f"{x:.4f}")
        else:
                return f"{x:.4f}"






# -*- encoding: utf-8 -*-
"""
@File    :   ops.py
@Time    :   2025/2/25 下午8:23
@Author  :   Li Jiawei
@Version :   1.0
@Contact :   Li.J.W.adrian421@hotmail.com
@License :   (C)Copyright 2023-2030
@Desc    :   None
@Brief   :
"""
import os, sys, argparse
from tqdm import tqdm
from itertools import groupby, tee
from datetime import timedelta
from collections import Counter
import codecs, json, \
        time, datetime, pickle, \
        re, random, numpy as np, \
        pandas as pd, xml.dom.minidom as xmldom, string
import inspect
import platform
import yaml
import itertools
from pdb import set_trace as stop
from typing import Any, List, Tuple, Union
from algo import groupby_continuous_index


def clearDir(*args):
        os.system("""find . -name "__pycache__"|xargs rm -rf""")
        os.system("""find . -name ".DS_Store"|xargs rm -rf""")
        for pattern in [*args]:
                os.system(f"""find . -name "{pattern}"|xargs rm -rf""")


def jdir(dirpath):
        os.makedirs(dirpath, exist_ok=True)
        return dirpath


def jpath(*args):
        full_path = os.path.join(*args)
        dirs = full_path.split('/')[:-1] if '/' in full_path else (full_path.split('\\')[:-1] if '\\' in full_path else '')
        dirs = [x for x in dirs if x]
        if dirs:
                for i in range(len(dirs)):
                        dir_path = jdir('/' + '/'.join(dirs[:i + 1]))
        return full_path


def say(txt):
        print(f"『 {txt} 』")
        return f"『 {txt} 』"


def show_rows(row):
        for idx, item in enumerate(row):
                print(f"[{idx}] {item}")


def db(expression, showValue=True, showUnique=False):
        '''
        :param expression: format: 'variable' or "variable"
        :param showValue: bool,whether show variable's content, if False, simply show the type and shape
        :return: None
        '''
        frame = sys._getframe(1)
        var = eval(expression, frame.f_globals, frame.f_locals)
        print('\n')
        print('-' * 25)
        print('[NAME] : %s' % expression)
        print("[TYPE] :", type(var))
        try:
                print("[SHAPE]:", var.shape)
                try:
                        var1 = var.flatten()
                        print(f"[STAT]: mean = {np.mean(var1):.4f}, min = {np.min(var1):.4f}, max = {np.max(var1):.4f}, median = {np.median(var1):.4f}, std={np.std(var1):.4f}, unique_num = {len(list(set(var1.tolist())))}")
                        if showUnique:
                                print(f"[UNIQUE VAL]: {list(set(var1.tolist()))}")
                except:
                        pass
        except:
                pass
        try:
                print("[LEN] :", len(var))
                try:
                        print(f"[STAT]: mean = {np.mean(var):.4f}, min = {np.min(var):.4f}, max = {np.max(var):.4f}, median = {np.median(var):.4f}, std={np.std(var):.4f}, unique_num = {len(list(set(var)))}")
                        if showUnique:
                                print(f"[UNIQUE VAL]: {list(set(var1))}")
                except:
                        pass
        except:
                pass
        if showValue:
                print('[VALUE]:', repr(var))
        if isinstance(eval(expression, frame.f_globals, frame.f_locals), dict):
                print('[KEYS] :', list(var.keys()))
        print('-' * 25)


def current_method_name():
        '''获取正在运行函数(或方法)名称'''
        return inspect.stack()[1][3]


def testbug(code, var):
        try:
                exec(code)
        except:
                raise ValueError(f"[CODE] try executing「{code}」 Failed! var is {var}")


def pairwise(it):
        """
        获取元素间的顺序对

        Parameter:
        ----------
        it: 列表或者迭代器，例如(s0,s1,s2,...,sn)

        Return:
        -------
        pair: (s0,s1),(s1,s2),(s2, s3),...,(sn-1, sn)
        """
        a, b = tee(it)
        next(b, None)
        return zip(a, b)


def inverse_dict(my_dict):
        """
        Get the key and values swapped new dict from an old dict

        Parameter:
        ----------
        my_dict: dict, {k:v}

        Return:
        -------
        {v:k}
        """
        return {v: k for k, v in my_dict.items()}


def flatten_multi_list(lst):
        return list(itertools.chain.from_iterable(lst))


def tqdmenum(x, desc):
        return tqdm(enumerate(x), total=len(x), desc=f"{desc}")


def mapfn(lambda_expression, lst):
        return list(map(lambda_expression, lst))


def fold_list(lst):
        """
        example:
        inputs:
                 [1, 2, 1, 2, 1, 2, 3, 4, 4, 4, 5, 6, 6, 7, 6, 7, 9, 9, 9, 9, 1, 2, 3, 4]
        outputs:
                 [1, 2, 1, 2, 1, 2, 3, '4>r3', 5, '6>r2', 7, 6, 7, '9>r4', 1, 2, 3, 4]

        """
        lst = lst + [-999314525]  # magic number, used to avoid that our function misses the last token of lst
        label_lst = ['0']
        for idx in range(1, len(lst)):
                label = '1' if lst[idx - 1] == lst[idx] else '0'
                label_lst.append(label)
        outputs = []
        for group in return_group(label_lst):
                if group[0] == '1':
                        outputs.append('>r'.join([str(lst[group[1][0] - 1]), str(group[1][1] - group[1][0] + 2)]))
                if group[0] == '0':
                        outputs.extend(lst[group[1][0]:group[1][1]])
        return outputs


def unfold_list(lst):
        """
        example:
        inputs:
                [1, 2, 1, 2, 1, 2, 3, '4>r3', 5, '6>r2', 7, 6, 7, '9>r4', 1, 2, 3, 4]
        outputs:
                [1, 2, 1, 2, 1, 2, 3, 4, 4, 4, 5, 6, 6, 7, 6, 7, 9, 9, 9, 9, 1, 2, 3, 4]

        """
        outputs = []
        for item in lst:
                if isinstance(item, str) and '>r' in item:
                        outputs += [param_to_number(item.split('>r')[0])] * param_to_number(item.split('>r')[1])
                else:
                        outputs += [param_to_number(item)]
        return outputs


def simplyfy_list(lst):
        lst = groupby_continuous_index(lst)
        lst = fold_list(lst)
        outputs = []
        for item in lst:
                if isinstance(item, list) and not '>r' in item:
                        outputs += [f"{item[0]}>c{len(item)}"]
                else:
                        outputs += [item]
        return outputs


def recover_list(lst):
        """
        example:
            inputs:
                [1, 2, 1, 2, 1, 2, 3, 4, 4, 4, 4, 3, 2, 5, 6, 6, 7, 6, 7, 9, 9, 9, 9, 1, 2, 3, 4]
                ['1>c2', '1>c2', '1>c4', '4>r3', 3, 2, '5>c2', '6>c2', '6>c2', '9>r4', '1>c4']
            outputs:
                [1, 2, 1, 2, 1, 2, 3, 4, 4, 4, 4, 3, 2, 5, 6, 6, 7, 6, 7, 9, 9, 9, 9, 1, 2, 3, 4]
        """
        outputs = []
        for item in lst:
                if isinstance(item, str):
                        if '>c' in item:
                                outputs += [int(item.split('>c')[0]) + i for i in range(int(item.split('>c')[1]))]
                                continue
                        if '>r' in item:
                                outputs += [param_to_number(item.split('>r')[0])] * param_to_number(item.split('>r')[1])
                                continue
                        else:
                                outputs += [item]
                                continue
                else:
                        outputs += [item]
        return outputs


def rows2cols(rows):
        """
        Convert rows like [[1,2,3],[4,5,6]] into columns like
        [[1,4],[2,5],[3,6]]

        Parameter:
        ----------
        rows: list, forms like [[1,2,3],[4,5,6]]

        Return:
        -------
        List like [[1,4],[2,5],[3,6]]
        """
        return list(zip(*rows))


def param_is_number(s):
        try:
                int(s)
                return True
        except ValueError:
                pass
        try:
                float(s)
                return True
        except (TypeError, ValueError):
                pass
        return False


def param_to_number(s):
        try:
                return int(s)
        except ValueError:
                pass
        try:
                return float(s)
        except (TypeError, ValueError):
                pass
        return s


def find_declared(fpath, declaration_name):
        ''' customize declaration field

        example:

            cfg.py:

                def useful():
                    def decorator(fn):
                        return fn
                    return lambda fn: decorator(fn)

                由于python中@只用于函数装饰器，因此若要修饰变量，只能将变量函数化
                class Params(object):
                    def __init__(self):
                        self.name('Alex')
                        self.age(23)
                        self.score(100)
                    def name(self,name):
                        self.name = name
                    @useful
                    def age(self,age):
                        self.age = age
                    @useful
                    def score(self,score):
                        self.score = score

            main.py:

                def get_useful_param():
                    fields = find_declared("cfg.py","@useful")
        '''
        fields = []
        with open(fpath, 'r', encoding='utf-8') as f:
                line = f.readline()
                while line:
                        line = line.strip()
                        if line.startswith(f"{declaration_name}"):
                                line = f.readline()
                                target = re.findall('\s(.*?)\(', line)[0]
                                fields.append(target.strip())
                        line = f.readline()
                return fields
